package com.platform.yijia.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.google.gson.Gson;
import com.platform.yijia.param.request.RechargeBalance;
import com.platform.yijia.pojo.*;
import com.platform.yijia.service.ElectronicMembershipCardService;
import com.platform.yijia.service.GzhNewsTemplateService;
import com.platform.yijia.service.StationService;
import com.platform.yijia.utils.*;
import com.platform.yijia.utils.weixinapp.WxPushUtil;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateData;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/*
 * <Title> ElectronicMembershipCardController </Title>
 * <Description> 电子会员卡（储蓄卡）功能模块 </Description>
 * @Author JK
 * @Date 2021年3月22日
 */
@Controller
@RequestMapping("/demo")
public class ElectronicMembershipCardController {
    private static Logger logger =(Logger) LoggerFactory.getLogger(ElectronicMembershipCardController.class);

    @Resource
    private ElectronicMembershipCardService electronicMembershipCardService;
    @Resource
    private StationService stationService;
    @Resource
    private GzhNewsTemplateService gzhNewsTemplateService;
    @Resource
    private WxPushUtil wxPushUtil;

    //获取用户电子会员卡（储蓄卡）列表
    @RequestMapping(value = "/getElectronicCardList", method = RequestMethod.GET)
    @ResponseBody
    public String getElectronicCardList(@RequestParam Integer stationId, String unionId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(stationId !=null && unionId !=null){
            CustomerElectronicCard customerElectronicCard = new CustomerElectronicCard();
            customerElectronicCard.setStationId(stationId);
            customerElectronicCard.setUnionId(unionId);
            List<CustomerElectronicCard> electronicCardList = electronicMembershipCardService.getElectronicCardList(customerElectronicCard);
            resultData = ResultData.success(electronicCardList);
        }else {
            resultData = ResultData.error(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //用户充值电子会员卡（储蓄卡）生成充值记录
    @RequestMapping(value = "/rechargeBalance", method = RequestMethod.POST)
    @ResponseBody
    public String rechargeBalance(@RequestBody RechargeBalance rechargeBalance){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(rechargeBalance !=null){
            BigDecimal amt = new BigDecimal(rechargeBalance.getAmt());
            //String openId = rechargeBalance.getOpenId();    //公众号的openId
            Integer stationId = rechargeBalance.getStationId();
            //BigDecimal presentAmt = new BigDecimal(rechargeBalance.getPresentAmt());
            String payType = rechargeBalance.getPayType();
            String customerName = rechargeBalance.getCustomerName();
            String cardOilsType = rechargeBalance.getCardOilsType();
            String unionId = rechargeBalance.getUnionId();
            String stationName = rechargeBalance.getStationName();
            String stationCardEnabledFlag = electronicMembershipCardService.getStationCardEnabledFlag(stationId);
            //是否启用电子卡功能
            if(stationCardEnabledFlag.equals("1")){
                CustomerCardUseRecord customerCardUseRecord = new CustomerCardUseRecord();      //电子会员卡使用记录
                CustomerCardSetting customerCardSetting = new CustomerCardSetting();        //电子卡优惠规则设置
                customerCardSetting.setStationId(stationId);
                customerCardSetting.setCardOilsType(cardOilsType);
                customerCardSetting.setOrderBy("DESC");
                //获取用户要充值电子卡的优惠规则信息
                List<CustomerCardSetting> cardRechargeSettingList = electronicMembershipCardService.getCardRechargeSettingList(customerCardSetting);
                if(cardRechargeSettingList !=null && cardRechargeSettingList.size() ==1){
                    String discountTime = cardRechargeSettingList.get(0).getDiscountTime();
                    String[] dctime={};     //会员充值活动时间
                    if(discountTime.contains(",")){
                        dctime =discountTime.split(",");
                    }else {
                        dctime =new String[1];
                        dctime[0] = discountTime;
                    }
                    String currentTime =null;
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(new Date());   //获取当前时间
                    //循环优惠规则明细
                    for (CustomerCardSettingDetail cardDetail : cardRechargeSettingList.get(0).getCustomerCardSettingDetailList()){
                        if(amt.compareTo(cardDetail.getDiscountAmtEnd()) ==1 || amt.compareTo(cardDetail.getDiscountAmtEnd()) ==0){
                            //String isDiscountCoupon = cardRechargeSettingList.get(0).getIsDiscountCoupon();       //是否使用优惠券
                            //充值优惠类型
                            if(cardDetail.getSettingRuleType().equals("1")){ //按充值金额
                                BigDecimal presentAmt = cardDetail.getPresentAmt(); //当前赠送金额
                                if(cardRechargeSettingList.get(0).getDiscountTimeSetting().equals("1") && dctime !=null){
                                    String discountTimeType = cardRechargeSettingList.get(0).getDiscountTimeType();     //获取日期类型
                                    switch (discountTimeType){
                                        case "1":   //  1.按周来
                                            int day_Week = calendar.get(Calendar.DAY_OF_WEEK);
                                            day_Week= day_Week-1;     //减一原因：数据库存的下标从零开始
                                            currentTime = String.valueOf(day_Week);
                                            break;
                                        case "2":   //  2.每月的多少号
                                            int day_month = calendar.get(Calendar.DAY_OF_MONTH);
                                            day_month = day_month-1;        //减一原因：数据库存的下标从零开始
                                            currentTime = String.valueOf(day_month); //当前时间
                                            break;
                                        case "3":   //  3.固定日期
                                            currentTime =new SimpleDateFormat("yyyy-MM-dd").format(new Date());
                                            break;
                                    }
                                    for (String str : dctime){
                                        if(str.equals(currentTime)){  //如果当前时间在会员日活动时间内
                                            //计算会员日内的总赠送积分
                                            if(cardRechargeSettingList.get(0).getPresentScale() !=null){
                                                presentAmt = presentAmt.multiply(new BigDecimal(cardRechargeSettingList.get(0).getPresentScale())).add(presentAmt);
                                            }
                                            break;
                                        }
                                    }
                                }
                                customerCardUseRecord.setAmt(amt);
                                customerCardUseRecord.setPresentAmt(presentAmt);
                            }else if(cardDetail.getSettingRuleType().equals("2")){      //按充值金额的比例
                                BigDecimal presentAmt = amt.multiply(cardDetail.getPresentAmt().divide(new BigDecimal(100))); //当前赠送金额比例的赠送积分
                                if(cardRechargeSettingList.get(0).getDiscountTimeSetting().equals("1") && dctime !=null){
                                    String discountTimeType = cardRechargeSettingList.get(0).getDiscountTimeType();     //获取日期类型
                                    switch (discountTimeType){
                                        case "1":   //  1.按周来
                                            int day_Week = calendar.get(Calendar.DAY_OF_WEEK);
                                            day_Week= day_Week-1;     //减一原因：数据库存的下标从零开始
                                            currentTime = String.valueOf(day_Week);
                                            break;
                                        case "2":   //  2.每月的多少号
                                            int day_month = calendar.get(Calendar.DAY_OF_MONTH);
                                            day_month = day_month-1;        //减一原因：数据库存的下标从零开始
                                            currentTime = String.valueOf(day_month); //当前时间
                                            break;
                                        case "3":   //  3.固定日期
                                            currentTime =new SimpleDateFormat("yyyy-MM-dd").format(new Date());
                                            break;
                                    }
                                    for (String str : dctime){
                                        if(str.equals(currentTime)){  //如果当前时间在会员日活动时间内
                                            //计算会员日期内的总赠送积分
                                            if(cardRechargeSettingList.get(0).getPresentScale() !=null){
                                                presentAmt = presentAmt.multiply(new BigDecimal(cardRechargeSettingList.get(0).getPresentScale())).add(presentAmt);
                                            }
                                            break;
                                        }
                                    }
                                }
                                customerCardUseRecord.setAmt(amt);
                                customerCardUseRecord.setPresentAmt(presentAmt);
                            }
                            break;
                        }
                    }
                }else {
                    customerCardUseRecord.setAmt(amt);
                    customerCardUseRecord.setPresentAmt(new BigDecimal(0));
                }
                customerCardUseRecord.setUnionId(unionId);
                customerCardUseRecord.setStationId(stationId);
                customerCardUseRecord.setCardOilsType(cardOilsType);
                customerCardUseRecord.setStationName(stationName);
                customerCardUseRecord.setCustomerName(customerName);
                customerCardUseRecord.setUsageType("+");
                customerCardUseRecord.setPayType(payType);      //1表示微信
                customerCardUseRecord.setCreateTime(new Date());
                customerCardUseRecord.setStatus("0");

                CustomerElectronicCard c = new CustomerElectronicCard();    //电子会员卡
                c.setUnionId(unionId);
                c.setCardOilsType(cardOilsType);
                CustomerElectronicCard electronicCardInfoByUnionId = electronicMembershipCardService.getElectronicCardInfoByUnionId(c);
                if(electronicCardInfoByUnionId !=null && electronicCardInfoByUnionId.getCustomerNo() !=null){
                    customerCardUseRecord.setCustomerNo(electronicCardInfoByUnionId.getCustomerNo());
                }
                //订单规则 时间+6位随机数
                Random random = new Random();
                String str="";
                for(int i=0; i<6; i++){
                    str+=random.nextInt(10);
                }
                String orderNo = System.nanoTime()+str;
                customerCardUseRecord.setOrderNo(orderNo);
                electronicMembershipCardService.insetCardUseRecord(customerCardUseRecord);      //插入电子会员卡的使用记录
                resultData = ResultData.success(orderNo);
            }
        }
        return gson.toJson(resultData);
    }

    //用户充值电子会员卡（储蓄卡）拉起随行付支付接口
    @RequestMapping(value = "/rechargeBalanceSXFPay", method = RequestMethod.POST)
    @ResponseBody
    public String rechargeBalanceSXFPay(@RequestBody RechargeBalance rechargeBalance){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(rechargeBalance !=null){
            String amt = rechargeBalance.getAmt();
            String openId = rechargeBalance.getOpenId();    //公众号的openId
            Integer stationId = rechargeBalance.getStationId();
            String userType = rechargeBalance.getUserType();
            String subject = rechargeBalance.getSubject();
            String orderNo = rechargeBalance.getOrderNo();
            //将业务参数存放在 JSON 对象中
            JSONObject reqData = new JSONObject();
            Map<String, String> resultMap = stationService.getStationAppIdAndAppSecret(stationId);
            if(resultMap !=null && resultMap.containsKey("gzhAppId") && resultMap.containsKey("mno")){
                logger.info("==============================电子会员卡（储蓄卡）充值功能调用随行付接口================================");
                try {
                    if(userType.equals("1")){
                        //1 是公众号
                        reqData.put("payWay", "02"); //支付方式  02 公众号/服 务窗/js支付 03 小程序
                        reqData.put("userId", openId); //用户号 微信：openid； 支付宝：userid；银联：userid；微信&支付宝必传，银联js为非必传
                        reqData.put("subAppid", resultMap.get("gzhAppId")); //微信公众号
                        logger.info("支付子商户："+ resultMap.get("gzhAppId"));
                    }else if(userType.equals("2")){
                        reqData.put("payWay", "03"); //支付方式  02 公众号/服 务窗/js支付 03 小程序
                        reqData.put("userId", openId); //用户号 微信：openid； 支付宝：userid；银联：userid；微信&支付宝必传，银联js为非必传
                        reqData.put("subAppid", resultMap.get("appId")); //微信小程序
                        logger.info("支付子商户："+ resultMap.get("appId"));
                    }
                    reqData.put("mno", resultMap.get("mno")); //商户编号
                    logger.info("商户编号："+ resultMap.get("mno"));
                    reqData.put("ordNo", orderNo); //商户订单号
                    reqData.put("amt", amt); //订单总金额
                    reqData.put("payType", "WECHAT"); //支付渠道
                    reqData.put("subject", subject); //订单标题
                    reqData.put("tradeSource", "01"); //交易来源 01服务商，02收银台，03硬件
                    reqData.put("trmIp", "172.16.2.1");//商家ip地址
                    reqData.put("outFrontFailUrl", ""); //js 支付，前台 事变通知地址
                    reqData.put("notifyUrl", "https://www.huijy.net/demo/getCardRechargeCallback"); //回调地址
//                    reqData.put("notifyUrl", "https://www.huijy.net/api/getCardRechargeCallback"); //回调地址
//                    reqData.put("subMechId", ""); //子商户号
//                    reqData.put("amt", "0.01");
//                    reqData.put("discountAmt", ""); //参与优惠金额
//                    reqData.put("unDiscountAmt", ""); //不参与优惠金额
//                    reqData.put("timeExpire", "10"); //订单失效时间, 以分钟为单位
//                    reqData.put("limitPay", "00"); //限制卡类型: 00-全部 01-限定不能使 用信用卡支付 默认值 00
//                    reqData.put("hbFqNum", "6"); //花呗分期数,仅可上送 6 或 12
//                    reqData.put("customerIp", ""); //持卡人ip地址，银联js支付时必传
//                    reqData.put("outFrontUrl", ""); //js 支付，前台 成功通知地址
//                    reqData.put("needReceipt", "00"); //电子发票功能 微信开具电子 发票使用
//                    reqData.put("ledgerAccountFlag", "00"); //是否做分账 分账交易使 用；00：做； 01：不做；不传默认为不做分账
//                    reqData.put("ledgerAccountEffectTime", "00"); //分账有效时间 单位为天；是 否做分账选择 00 时该字段必传
//                    reqData.put("ruleId", "00"); //同 步 分 账 规 则 id
//                    reqData.put("fusruleId", "00"); //同步分账规则
//                    reqData.put("ylTrmNo", ""); //银联终端号
//                    reqData.put("terminalId", ""); //TQ机具编号
//                    reqData.put("deviceNo ", "1111"); //设备号
//                    reqData.put("identityFlag", ""); //是否是实名支付
//                    reqData.put("buyerIdType", "IDCARD"); //证件类型
//                    reqData.put("buyerIdNo", "410523198701054018"); //证件号
//                    reqData.put("buyerName", "张三"); //买家姓名
//                    reqData.put("mobileNum", ""); //手机号
//                    reqData.put("extend", ""); //备用
                    String req= SXFPaySign.getSXFPay(reqData);
                    logger.debug("参数信息："+req);
                    System.out.println("req:" + req);
                    //此处不要改变reqData里面值的顺序用LinkedHashMap
                    HashMap reqMap = JSON.parseObject(req, LinkedHashMap.class, Feature.OrderedField);
                    //组装加密串
                    String signContent = RSASignature.getOrderContent(reqMap);
                    System.out.println("拼接后的参数：" + signContent);
                    logger.debug("拼接后的参数："+signContent);
                    //sign
                    String sign = RSASignature.encryptBASE64(RSASignature.sign(signContent, SXFPaySign.privateKey));
                    System.out.println("============签名:" + sign);
                    reqMap.put("sign", sign);
                    String reqStr = JSON.toJSONString(reqMap);
                    System.out.println("请求参数:" + reqMap);
                    System.out.println("请求参数:" + reqStr);
                    //调用第三方接口地址
                    //String url = "https://openapi-test.tianquetech.com/order/jsapiScan";//测试地址
                    String url = "https://openapi.suixingpay.com/order/jsapiScan";//生产地址
                    String resultJson = HttpUtils.connectPostUrl(url, reqStr);
                    System.out.println("返回信息：" + resultJson);
                    //   logger.debug("返回信息："+resultJson);
                    //不要对reqData排序 所以用LinkedHashMap
                    HashMap<String, Object> result = JSON.parseObject(resultJson, LinkedHashMap.class, Feature.OrderedField);
                    if ("0000".equals(result.get("code"))) {
                        //验签
                        String signResult = result.get("sign").toString();
                        result.remove("sign");
                        String resultStr = RSASignature.getOrderContent(result);
                        System.out.println(resultStr);
                        //sign
                        String resultSign = RSASignature.encryptBASE64(RSASignature.sign(signContent, SXFPaySign.privateKey));
                        System.out.println("resultSign:" + resultSign);
                        //组装加密串
                        if (RSASignature.doCheck(resultStr, signResult, SXFPaySign.sxfPublic)) {
                            System.out.println("===================验签成功==============");
                        }
                    }
                    resultData=ResultData.success(result);
                }catch (Exception e){
                    resultData=ResultData.error(CodeMsg.YEWU_FAIL,"订单支付失败");
                    e.printStackTrace();
                }
            }
        }else {
            resultData = ResultData.error(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //用户电子会员卡（储蓄卡）充值功能随行付回调功能
    @RequestMapping(value = "/getCardRechargeCallback", consumes = "application/json", method = RequestMethod.POST)
    @ResponseBody
    @Transactional
    public void getCardRechargeCallback(@RequestBody JSONObject request){
        logger.info("=========================电子会员卡（储蓄卡）充值功能随行付回调功能===============================");
        logger.info("电子会员卡（储蓄卡）充值功能随行付回调返回过来的参数："+request);
        //logger.info("ordNo信息：" + request.get("ordNo").toString());
        //logger.info("sign信息：" + request.get("sign").toString());
        Gson gson =new Gson();
        String sxfUuid=request.get("sxfUuid").toString();     //随行付落单号
        String payWay=request.get("payWay").toString();     //随行付交易方式
        String payTime=request.get("payTime").toString();   //随行付交易时间
        String uuid=request.get("uuid").toString();         //随行付交易流水号
        String mno=request.get("mno").toString();//发起交易的商户编号
        String ordNo=request.get("ordNo").toString();//商户生成的订单号
        String amt=request.get("amt").toString();//金额
        String buyerId=request.get("buyerId").toString();//用户登录
        String transactionId=request.get("transactionId").toString();//用户登录
        String drType=request.get("drType").toString();//借贷记标识 1-借,2-贷，3-其他
        String payType=request.get("payType").toString();//对订单的描述，取值范围：WECHAT:微信,ALIPAY:支付宝,UNIONPAY:
        String payBank=request.get("payBank").toString();//付款银行银行编码，例如 ICBC
        String pointAmount=request.get("pointAmount").toString();//代金券金额
        String totalOffstAmt=request.get("totalOffstAmt").toString();//消 费 者 付 款金额
        String settleAmt=request.get("settleAmt").toString();//商家入账金额
        String timeStamp=request.get("timeStamp").toString();//响应时间
        String sign=request.get("sign").toString();//签名
        String bizCode=request.get("bizCode").toString();//业务返回码
        String bizMsg=request.get("bizMsg").toString();//业 务 返 回 信息
        String openid=request.get("openid").toString();//微 信 或 支 付宝身份id()opendid userid
        String channelId=request.get("channelId").toString();//渠道商户号
        String subMechId=request.get("subMechId").toString();//子商户号
        String finishTime=request.get("finishTime").toString();//交易完成时间
        String clearDt=request.get("clearDt").toString();
        String settlementBatchNo=request.get("settlementBatchNo").toString();
        //String origOrdNo=request.get("origOrdNo").toString();//反交易退款异步通知
        //String origUuid=request.get("origUuid").toString();//token
        //String scene=request.get("scene").toString();//用户登录
        //String buyerAccount=request.get("buyerAccount").toString();//
        //String realRefundAmount=request.get("realRefundAmount").toString();//商家出账金额
        //String recFeeAmt=request.get("recFeeAmt").toString();//交易手续费
        //String recFeeRate=request.get("recFeeRate").toString();//交易手续费率
        //String refBuyerAmt=request.get("refBuyerAmt").toString();//消 费 者 到 账金额
        //String extend = extend=request.get("extend").toString();//备用

        //将业务参数存放在 JSON 对象中
        JSONObject reqData = new JSONObject();
        //备用
        if(request.containsKey("extend") && request.get("extend") !=null){
            String extend = request.get("extend").toString();
            reqData.put("extend", extend);
        }
        //优惠券参数
        if(request.containsKey("promotionDetail") && request.get("promotionDetail") !=null){
            String promotionDetail = request.get("promotionDetail").toString();
            reqData.put("promotionDetail", promotionDetail);
        }
        //商家出账金额
        if(request.containsKey("realRefundAmount") && request.get("realRefundAmount") !=null){
            String realRefundAmount = request.get("realRefundAmount").toString();
            reqData.put("realRefundAmount", realRefundAmount);
        }
        //szltFlag
        if(request.containsKey("szltFlag") && request.get("szltFlag") !=null){
            String szltFlag = request.get("szltFlag").toString();
            reqData.put("szltFlag", szltFlag);
        }
        //recFeeAmt
        if(request.containsKey("recFeeAmt") && request.get("recFeeAmt") !=null){
            String recFeeAmt = request.get("recFeeAmt").toString();
            reqData.put("recFeeAmt", recFeeAmt);
        }
        //szltRecfeeAmt
        if(request.containsKey("szltRecfeeAmt") && request.get("szltRecfeeAmt") !=null){
            String szltRecfeeAmt = request.get("szltRecfeeAmt").toString();
            reqData.put("szltRecfeeAmt", szltRecfeeAmt);
        }

        reqData.put("sxfUuid",sxfUuid);
        reqData.put("payWay",payWay);
        reqData.put("payTime",payTime);
        reqData.put("uuid",uuid);
        reqData.put("mno",mno);
        reqData.put("ordNo",ordNo);
        reqData.put("amt",amt);
        reqData.put("buyerId",buyerId);
        reqData.put("transactionId",transactionId);
        reqData.put("drType",drType);
        reqData.put("payType",payType);
        reqData.put("payBank",payBank);
        reqData.put("pointAmount",pointAmount);
        reqData.put("totalOffstAmt",totalOffstAmt);
        reqData.put("settleAmt",settleAmt);
        reqData.put("timeStamp",timeStamp);
        reqData.put("bizCode",bizCode);
        reqData.put("bizMsg",bizMsg);
        reqData.put("openid",openid);
        reqData.put("channelId",channelId);
        reqData.put("subMechId",subMechId);
        reqData.put("finishTime",finishTime);
        reqData.put("clearDt",clearDt);
        reqData.put("settlementBatchNo",settlementBatchNo);
        //reqData.put("realRefundAmount",realRefundAmount);
        //reqData.put("recFeeAmt",recFeeAmt);
        //reqData.put("recFeeRate",recFeeRate);
        //reqData.put("buyerAccount",buyerAccount);
        //reqData.put("origOrdNo",origOrdNo);
        //reqData.put("origUuid",origUuid);
        //reqData.put("scene",scene);
        //reqData.put("refBuyerAmt",refBuyerAmt);
        //reqData.put("extend",extend);
        //根据回调信息，先进行验签，验签后修改状态
        logger.info("打印参数信息"+gson.toJson(reqData));
        //不要对reqData排序 所以用LinkedHashMap
        HashMap<String, Object> result = JSON.parseObject(gson.toJson(reqData), LinkedHashMap.class, Feature.OrderedField);
        //验签
        String resultStr = RSASignature.getOrderContent(result);
        //JSONObject req = new JSONObject();
        System.out.println(resultStr);
        if (RSASignature.doCheck(resultStr, sign, SXFPaySign.sxfPublic)) {
            logger.info("验签成功");
            //随行付回调验签成功更新客户电子卡余额 和 更新客户充值记录状态
            CustomerCardUseRecord customerCardUseRecord = new CustomerCardUseRecord();
            customerCardUseRecord.setOrderNo(ordNo);
            CustomerCardUseRecord customerCardUseRecordInfo = electronicMembershipCardService.getCardRecordByOrderNo(customerCardUseRecord);
            // 0表示未支付  1表示已支付
            if(customerCardUseRecordInfo !=null && customerCardUseRecordInfo.getStatus().equals("0")){
                logger.info("充值单号：" + ordNo);
                CustomerElectronicCard customerElectronicCard = new CustomerElectronicCard();
                customerElectronicCard.setUnionId(customerCardUseRecord.getUnionId());
                customerCardUseRecord.setStationId(customerCardUseRecordInfo.getStationId());
                //是否有赠送金额
                BigDecimal presentAmt =new BigDecimal(0);
                if(customerCardUseRecordInfo.getPresentAmt() !=null){
                    presentAmt = customerCardUseRecordInfo.getPresentAmt();  //客户充值赠送金额
                    customerElectronicCard.setAmt(new BigDecimal(amt).add(presentAmt));
                }else {
                    customerElectronicCard.setAmt(new BigDecimal(amt));
                }
                customerElectronicCard.setRecentlyTime(new Date());
                customerElectronicCard.setCardOilsType(customerCardUseRecordInfo.getCardOilsType());
                electronicMembershipCardService.updateElectronicCardInfoByUnionId(customerElectronicCard);
                customerCardUseRecord.setStatus("1");
                customerCardUseRecord.setBalance(customerElectronicCard.getAmt());
                electronicMembershipCardService.updateCardRecordStatusByOrderNo(customerCardUseRecord);
                logger.info("=====================电子会员卡充值成功=============================");
                customerCardUseRecord.setUnionId(customerCardUseRecordInfo.getUnionId());
                CustomerElectronicCard electronicCardInfo = electronicMembershipCardService.getElectronicCardInfo(customerElectronicCard);
                if(electronicCardInfo !=null && electronicCardInfo.getBlogOpenid() !=null){
                    String stationName = electronicCardInfo.getStationName();
                    String blogOpenid = electronicCardInfo.getBlogOpenid();
                    Integer stationId = electronicCardInfo.getStationId();      //油站ID
                    String customerName = electronicCardInfo.getCustomerName(); //客户名称
                    BigDecimal balance = electronicCardInfo.getAmt();   //当前余额
                    String customerNo = electronicCardInfo.getCustomerNo();     //电子卡号
                    //获取要推送充值成功模板的ID
                    GzhNewsTemplate gzhNewsTemplate = new GzhNewsTemplate();
                    gzhNewsTemplate.setTemplateType("3");   //3 代表电子会员卡充值成功模板类型
                    gzhNewsTemplate.setStationId(stationId);
                    GzhNewsTemplate gzhNewsTemplateInfo = gzhNewsTemplateService.getGzhNewsTemplate(gzhNewsTemplate);
                    if(gzhNewsTemplateInfo !=null && gzhNewsTemplateInfo.getTemplateId() !=null){
                        List<WxMpTemplateData> wxMpTemplate = new ArrayList<>();
                        wxMpTemplate.add(new WxMpTemplateData("first","尊敬的"+customerName+"，您好：\n" + "您在"+stationName+"的电子会员卡充值成功信息如下"));
                        wxMpTemplate.add(new WxMpTemplateData("keyword1", customerNo));
                        wxMpTemplate.add(new WxMpTemplateData("keyword2", amt));
                        wxMpTemplate.add(new WxMpTemplateData("keyword3", presentAmt.toString()));
                        wxMpTemplate.add(new WxMpTemplateData("keyword4", balance.toString()));
                        wxMpTemplate.add(new WxMpTemplateData("keyword5", new SimpleDateFormat("yyyy年MM月dd日HH时mm分").format(new Date())));
                        wxMpTemplate.add(new WxMpTemplateData("remark","感谢您在"+ stationName+"的本次充值！"));
                        Map<String, String> m = stationService.getStationAppIdAndAppSecret(stationId);
                        if(m !=null && m.containsKey("gzhAppId") && m.containsKey("gzhAppSecret")){
                            String gzhAppId = m.get("gzhAppId");
                            String gzhAppSecret = m.get("gzhAppSecret");
                            wxPushUtil.push(gzhAppId, gzhAppSecret, gzhNewsTemplateInfo.getTemplateId(), blogOpenid, wxMpTemplate);
                            logger.info("电子会员卡公众号消息推送成功，推送人："+ blogOpenid);
                        }

                    }
                }
            }
        }else {
            logger.info("验签失败");
        }
    }

    //获取用户电子会员卡（储蓄卡）充值记录
    @RequestMapping(value = "/getCardRechargeRecord", method = RequestMethod.GET)
    @ResponseBody
    public String getCardRechargeRecord(@RequestParam Integer stationId, String unionId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(stationId !=null && unionId !=null){
            CustomerCardUseRecord customerCardUseRecord = new CustomerCardUseRecord();
            customerCardUseRecord.setUsageType("+"); // '+' 代表充值
            customerCardUseRecord.setStationId(stationId);
            customerCardUseRecord.setUnionId(unionId);
            List<CustomerCardUseRecord> cardUseRecordList = electronicMembershipCardService.getCardUseRecordList(customerCardUseRecord);
            resultData = ResultData.success(cardUseRecordList);
        }else {
            resultData = ResultData.error(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //获取用户电子会员卡（储蓄卡）消费记录
    @RequestMapping(value = "/getCardConsumptionRecord", method = RequestMethod.GET)
    @ResponseBody
    public String getCardConsumptionRecord(@RequestParam Integer stationId, String unionId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(stationId !=null && unionId !=null){
            CustomerCardUseRecord customerCardUseRecord = new CustomerCardUseRecord();
            customerCardUseRecord.setUnionId(unionId);
            customerCardUseRecord.setStationId(stationId);
            customerCardUseRecord.setUsageType("-"); // '-' 代表消费
            List<CustomerCardUseRecord> cardUseRecordList = electronicMembershipCardService.getCardUseRecordList(customerCardUseRecord);
            resultData = ResultData.success(cardUseRecordList);
        }else {
            resultData = ResultData.error(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //查询油站电子会员卡（储蓄卡）充值设置规则
    @RequestMapping(value = "/getCardRechargeSettingList", method = RequestMethod.GET)
    @ResponseBody
    public String getCardRechargeSettingList(@RequestParam Integer stationId, String cardOilsType){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(stationId !=null && StringUtils.isNotBlank(cardOilsType)){
            CustomerCardSetting customerCardSetting = new CustomerCardSetting();
            customerCardSetting.setStationId(stationId);
            customerCardSetting.setCardOilsType(cardOilsType);
            customerCardSetting.setOrderBy("ASC");  //优惠规则升序排序
            List<CustomerCardSetting> cardRechargeSettingList = electronicMembershipCardService.getCardRechargeSettingList(customerCardSetting);
            resultData = ResultData.success(cardRechargeSettingList);
        }else {
            resultData = ResultData.error(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

}
